import pygame
import os
import math
import level
import input
import player
import data

class game:
    """The game class is responsible for updating all objects."""
    def __init__(self, screen):
        self.screen = screen
        self.level = level.level('testlevel.txt', 640, 480)
        self.input = input.input()
        self.playersprites = pygame.sprite.RenderPlain()
        self.bulletsprites = pygame.sprite.RenderPlain()
        self.projectionsprites = pygame.sprite.RenderPlain()
        self.player = player.player(self.level.playerpos, 'playeravatar', self.level.camerax, self.projectionsprites, self.bulletsprites)
        self.playersprites.add(self.player)
        self.level.registerPlayer(self.player)
        self.font = pygame.font.SysFont('Helvetica', 48, False, False)
        self.healthbar = data.loadimage('health')
        self.healthbar.set_colorkey((0,0,0))
        self.remaininghealth = pygame.Surface((self.player.health, 20))
        self.remaininghealth.fill((255, 0, 0))
        
    def draw(self):
        self.level.draw(self.screen)
        self.playersprites.draw(self.screen)
        self.bulletsprites.draw(self.screen)
        if self.player.projected:
            b = pygame.Surface((640, 480), pygame.locals.SRCALPHA)
            b.fill((128,128,128,128))
            self.screen.blit(b, (0, 0))
            startpos = (self.player.rect.centerx, self.player.rect.centery)
            endpos = (self.player.projection.rect.centerx, self.player.projection.rect.centery)
            pygame.draw.line(self.screen, (255,255,255), startpos, endpos, 3)
            self.projectionsprites.draw(self.screen)
            self.player.projection.bullets.draw(self.screen)
        else:
            #health = self.font.render(self.player.health.__str__(), False, (255, 0, 0))
            #self.screen.blit(health, (10, 10))
            self.screen.blit(self.healthbar, (10, 10))
            self.screen.blit(self.remaininghealth, (30, 20))
            
        if not self.player.alive():
            print 'dead'
            b = pygame.Surface((640, 480), pygame.locals.SRCALPHA)
            b.fill((100,100,100,200))
            self.screen.blit(b,(0,0))    
            text = self.font.render('Game Over', False, (255, 0, 0))
            self.screen.blit(text, (200, 200))
            
    def update(self, elapsed):
        if self.player.projected:
            self.projectedUpdate(elapsed)
        else:
            self.normalUpdate(elapsed)
        
        self.remaininghealth = pygame.Surface((self.player.health, 20))
        self.remaininghealth.fill((255,0,0))
        
    def projectedUpdate(self, elapsed):
        
        if self.input.getState()['right']:
            self.player.projection.moveRight()
        if self.input.getState()['left']:
            self.player.projection.moveLeft()
        if self.input.getState()['up']:
            self.player.projection.moveUp()
        if self.input.getState()['down']:
            self.player.projection.moveDown()
        
        for s in self.level.souls.sprites():
            if self.player.rect.centery < self.player.projection.rect.centery:
                if s.rect.bottom > self.player.rect.centery and s.rect.top < self.player.projection.rect.centery:
                    if self.player.rect.centerx < self.player.projection.rect.centerx:
                        if s.rect.right > self.player.rect.centerx and s.rect.left < self.player.projection.rect.centerx:
                            if math.fabs(self.player.projection.perpVec.dot(s.vector)- self.player.projection.perpVec.dot(self.player.vector)) < 20:# and s.vector.dot(self.player.projection.normVec) >= self.player.vector.dot(self.player.projection.normVec) and s.vector.dot(self.player.projection.normVec) < self.player.projection.vector.dot(self.player.projection.normVec):
                                print 'collided'
                                
                            
                    else:
                        if s.rect.right > self.player.projection.rect.centerx and s.rect.left < self.player.rect.centerx:
                            if math.fabs(self.player.projection.perpVec.dot(s.vector)- self.player.projection.perpVec.dot(self.player.vector)) < 20:# and s.vector.dot(self.player.projection.normVec) >= self.player.vector.dot(self.player.projection.normVec) and s.vector.dot(self.player.projection.normVec) < self.player.projection.vector.dot(self.player.projection.normVec):
                                print 'collided'
            else:
                if s.rect.bottom > self.player.projection.rect.centery and s.rect.top < self.player.rect.centery:
                    if self.player.rect.centerx < self.player.projection.rect.centerx:
                        if s.rect.right > self.player.rect.centerx and s.rect.left < self.player.projection.rect.centerx:
                            if math.fabs(self.player.projection.perpVec.dot(s.vector)- self.player.projection.perpVec.dot(self.player.vector)) < 20:# and s.vector.dot(self.player.projection.normVec) >= self.player.vector.dot(self.player.projection.normVec) and s.vector.dot(self.player.projection.normVec) < self.player.projection.vector.dot(self.player.projection.normVec):
                                print 'collided'
                    else:
                        if s.rect.right > self.player.projection.rect.centerx and s.rect.left < self.player.rect.centerx:
                            if math.fabs(self.player.projection.perpVec.dot(s.vector)- self.player.projection.perpVec.dot(self.player.vector)) < 20:# and s.vector.dot(self.player.projection.normVec) >= self.player.vector.dot(self.player.projection.normVec) and s.vector.dot(self.player.projection.normVec) < self.player.projection.vector.dot(self.player.projection.normVec):
                                print 'collided'
        
        for b in self.player.projection.bullets.sprites():
            if b.rect.right < 0 or b.rect.left > 640 or b.rect.bottom < 0 or b.rect.top > 480:
                b.kill()
            
            for s in self.level.souls:
                if s.rect.colliderect(b.rect):
                    b.kill()
                    s.kill()
                
        if self.input.getState()['attack']:
            self.player.projection.attack(self.input.getState()['pos'])
                               
        if self.input.getState()['project']:
            self.player.project()
        
        self.playersprites.update(self.level.camerax, elapsed)
        self.projectionsprites.update(elapsed)
        self.level.update(elapsed)
        
    def normalUpdate(self, elapsed):
        if self.player.alive():
            if self.input.getState()['right']:
                if not self.level.checkCollision(self.player.pos, level.RIGHT, self.player.hvelocity):
                    if not self.level.checkSoulWall(self.player.pos):
                        if self.level.camerax < (self.level.levelw * 32) - 640:
                            if self.player.rect.centerx < 320:
                                self.player.moveRight(True)
                            else:
                                self.player.moveRight(False)
                                self.level.camerax += self.player.hvelocity
                                if self.level.camerax > (self.level.levelw * 32) - 640:
                                    self.level.camerax = (self.level.levelw * 32) - 640
                        else:
                            if self.player.rect.right < 640:
                                self.player.moveRight(True)
                else:
                    offset = (self.player.pos.right - 1 + self.player.hvelocity) % 32
                    self.player.pos.right = (self.player.pos.right - 1 + self.player.hvelocity) - offset
                    
            if self.input.getState()['left']:
                if not self.level.checkCollision(self.player.pos, level.LEFT, self.player.hvelocity):
                    if self.level.camerax > 0:
                        if self.player.rect.centerx > 320:
                            self.player.moveLeft(True)
                        else:
                            self.player.moveLeft(False)
                            self.level.camerax -= self.player.hvelocity
                            if self.level.camerax < 0:
                                self.level.camerax = 0
                    else:
                        if self.player.rect.left > 0:
                            self.player.moveLeft(True)
                else:
                    offset = 32 - ((self.player.pos.left - self.player.hvelocity) % 32)
                    self.player.pos.left = (self.player.pos.left - self.player.hvelocity) + offset
                    
            if self.input.getState()['up']:
                if not self.level.checkCollision(self.player.pos, level.UP, -self.player.jumpvel):
                    self.player.jump()
            
            if self.player.jumping:
                if self.level.checkCollision(self.player.pos, level.UP, -self.player.jumpvel):
                    self.player.hitFall()
            if not self.level.checkCollision(self.player.pos, level.DOWN, self.player.fallvel):
                self.player.fall()
            else:
                self.player.ground()
            
            for b in self.bulletsprites.sprites():
                if self.level.checkCollision(b.pos, level.RIGHT, b.velocity):
                    b.kill()
                if self.level.checkCollision(b.pos, level.LEFT, b.velocity):
                    b.kill()
                
                for e in self.level.enemies.sprites():
                    if e.pos.colliderect(b.pos):
                        b.kill()
                        e.hit(b.damage, self.level)
                    
            for e in self.level.enemies.sprites():
                if e.pos.colliderect(self.player.pos):
                    self.player.hit(e.damage)                
                    e.blocked()
                    
            if self.input.getState()['attack']:
                self.player.attack(self.level.camerax)
            
            if self.level.atSoulWall:
                if self.input.getState()['project']:
                    self.player.project()
                
            self.playersprites.update(self.level.camerax, elapsed)   
            self.bulletsprites.update(self.level.camerax, elapsed) 
            self.level.update(elapsed)
                
if __name__ == "__main__":
    go = game(None)