import pygame
import data
import enemy
import soul
import soulWall

UP = 1
LEFT = 2
DOWN = 3
RIGHT = 4

class level:
    """Loads and draws a game level"""
    def __init__(self, levelname, screenw, screenh):
        
        self.level = []
        self.leveldata = []
        self.tilesize = 32
        self.screenw = screenw
        self.screenh = screenh
        self.playerpos = (0,0)
        self.camerax = 0
        self.cameray = 0
        self.enemies = pygame.sprite.Group()
        self.souls = pygame.sprite.Group()
        self.player = None
        self.aliveSouls = 0
        self.atSoulWall = False
        self.soulWalls = []
        self.wallsGroup = []
        
        for x in xrange(10):
            s = soul.soul(self.souls, self)
        #map load begin
        file = data.loadtext(levelname, "r")
        line = file.readline()
        wh = line.split('x')
        self.levelw = int(wh[0])
        self.levelh = int(wh[1])
        
        self.level = [[0 for col in range(self.levelw)] for row in range(self.levelh)]
        self.leveldata = [[0 for col in range(self.levelw)] for row in range(self.levelh)]
        
        self.tilesetname = file.readline().rstrip()
        
        for i in range(self.levelh):
            line = file.readline().rstrip()
            for j in range(self.levelw):
                self.level[i][j] = int(line[j])
        #map load end
        
        #tileset load begin
        tilesetimage = data.loadimage(self.tilesetname) 
        self.tileset = []
        xtiles = tilesetimage.get_rect().width / self.tilesize
        ytiles = tilesetimage.get_rect().height / self.tilesize
        
        for i in range(xtiles*ytiles):
            self.tileset.append(pygame.Surface((self.tilesize,self.tilesize)))
        
        for y in range(ytiles):
            for x in range(xtiles):
                self.tileset[x + (y * xtiles)].blit(tilesetimage,(0,0), ((x*self.tilesize, y*self.tilesize, self.tilesize, self.tilesize)))
        #tileset load end
        
        line = file.readline()

        #map data load begin
        for i in range(self.levelh):
            line = file.readline().rstrip()
            for j in range(self.levelw):
                self.leveldata[i][j] = int(line[j])
        #map data load end
        
        y = 0
        for row in self.leveldata:
            x = 0
            for col in row:
                if col == 1:
                    self.playerpos = (x * self.tilesize, (y * self.tilesize) + 32)
                    if self.playerpos[0] >= 640:
                        self.camerax = self.playerpos - 128
                elif col == 3:
                    pos = (x * self.tilesize, (y * self.tilesize) + 32)
                    self.enemies.add(enemy.enemy(pos, 'entity', self.camerax, self.souls))
                elif col == 4:
                    self.soulWalls.append(x)
                x += 1
            y += 1
        
        for pos in self.soulWalls:
            w = soulWall.soulWall(pos*32)
            self.wallsGroup.append(w)
            
        self.currentWall = self.wallsGroup.pop(0)    
        self.nextWall = self.soulWalls.pop(0)
        
    def draw(self, screen):
        starttilex = (self.camerax / self.tilesize) % self.levelw
        starttiley = (self.cameray / self.tilesize) % self.levelh
        startdrawx = -(self.camerax % self.tilesize)
        startdrawy = -(self.cameray % self.tilesize)
        
        tiley = starttiley
        drawy = startdrawy
        
        while (drawy < self.screenh): 
            tilex = starttilex
            drawx = startdrawx
            
            while (drawx < self.screenw):
                dest = pygame.Rect(0,0,0,0)
                dest.left = drawx
                dest.top = drawy
                dest.width = self.tilesize
                dest.height = self.tilesize
                screen.blit(self.tileset[self.level[tiley][tilex]], dest)
                tilex += 1
                tilex %= self.levelw
                drawx += self.tilesize
            tiley += 1
            tiley %= self.levelh
            drawy += self.tilesize
            
        self.enemies.draw(screen)
        
        screen.blit(self.currentWall.image, self.currentWall.rect)
        
        if self.player.projected:
            self.souls.draw(screen)
            
    def update(self, elapsed):
        
        self.currentWall.update(self.camerax, elapsed)
        for w in self.wallsGroup:
            w.update(self.camerax, elapsed)
        
        if self.player.projected:
            self.souls.update(self.camerax, elapsed, self.player)
        else:
            for e in self.enemies.sprites():
                if e.direction == enemy.RIGHT:
                    if not self.checkCollision(e.pos, RIGHT, e.hvelocity):
                        e.moveRight()
                    else:
                        e.blocked()
                        
                elif e.direction == enemy.LEFT:
                    if not self.checkCollision(e.pos, LEFT, e.hvelocity):
                        e.moveLeft()
                    else:
                        e.blocked()
                        
                if self.checkCliff(e.pos, e.fallvel, e.direction):
                    e.blocked()
                else:
                    e.ground()
                            
            self.enemies.update(self.camerax)
            print self.aliveSouls
            
            if self.aliveSouls <= 0: 
                
                self.currentWall = self.wallsGroup.pop(0)
                for x in xrange(10):
                    s = soul.soul(self.souls, self)
                    
                
        if ((self.camerax + 639) / 32) >= self.nextWall:
            self.atSoulWall = True
            if len(self.soulWalls) > 0:
                self.nextWall = self.soulWalls.pop(0)
        
            
    def checkCollision(self, rect, direction, vel):
        if direction == RIGHT:
            x = (rect.right - 1 + vel) / 32
            y1 = rect.top / 32
            y2 = (rect.centery - 1) / 32
            y3 = (rect.bottom - 1) / 32
                        
            if x >= self.levelw:
                return True
            
            if self.leveldata[y1][x] == 2 or self.leveldata[y2][x] == 2 or self.leveldata[y3][x] == 2:
                return True
                
        if direction == DOWN:
            x1 = rect.left / 32
            x2 = (rect.right - 1) / 32
            y = (rect.bottom - 1 + vel) / 32
            
            try:
                if self.leveldata[y][x1] == 2 or self.leveldata[y][x2] == 2:
                    return True
            except:
                pass
               
        if direction == LEFT:
            x = (rect.left - vel) / 32
            y1 = rect.top / 32
            y2 = (rect.centery - 1) / 32
            y3 = (rect.bottom -1) / 32
                        
            if x < 0:
                return True
            
            if self.leveldata[y1][x] == 2 or self.leveldata[y2][x] == 2 or self.leveldata[y3][x] == 2:
                return True
            
        if direction == UP:
            x1 = rect.left / 32
            x2 = (rect.right - 1) / 32
            y = (rect.top - vel) / 32
            if self.leveldata[y][x1] == 2 or self.leveldata[y][x2] == 2:
                return True
            
        return False
    
    def checkCliff(self, rect, vel, dir):
        x1 = rect.left / 32
        x2 = (rect.right - 1) /32
        y = (rect.bottom - 1 + vel) / 32
        
        if dir == LEFT:
            if self.leveldata[y][x1] <> 2:
                return True
        elif dir == RIGHT:
            if self.leveldata[y][x2] <> 2:
                return True
        
        return False
    
    def checkSoulWall(self, rect):
            if self.currentWall.pos.colliderect(rect):
                return True
            
    def registerPlayer(self, player):
        self.player = player